/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.javagui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * The new Calendar insertion panel.
 *
 * @version $Id: CalendarNew.java,v 1.5 2007/01/11 19:27:30 nichele Exp $
 */
public class CalendarNew
extends Panel
implements ActionListener, ConfigurationParameters {

    //------------------------------------------------------------- Private data

    //
    // The window containing this panel
    //
    private MainWindow          mw      = null;
    private CalendarForm        form    = null;
    private CheckCalendarFields cfields = null;
    private Language            ln      = new Language();

    //----------------------------------------------------------- Public methods

    /**
     * Creates the panel.
     *
     * @param mw the window containing this panel
     */
    public CalendarNew(MainWindow mw) {

        Label  title       = null;
        Button butOk       = null;
        Button butCancel   = null;
        Panel  buttonPanel = null;

        this.mw = mw;

        setLayout(new BorderLayout());
        title = new Label(ln.getString("new_calendar"));

        form = new CalendarForm(mw);

        butOk = new Button(ln.getString("ok"));
        butOk.setActionCommand("ok");
        butOk.addActionListener(this);

        butCancel = new Button(ln.getString("cancel"));
        butCancel.setActionCommand("cancel");
        butCancel.addActionListener(this);

        buttonPanel = new Panel();
        buttonPanel.setLayout(new GridLayout(1, 2));
        buttonPanel.add(butOk    );
        buttonPanel.add(butCancel);

        add(title      , BorderLayout.NORTH );
        add(form       , BorderLayout.CENTER);
        add(buttonPanel, BorderLayout.SOUTH );
    }

    /**
     * Invoked when an action occurs (i.e. a button is pressed).
     *
     * @param evt the occurred action
     */
    public void actionPerformed(ActionEvent evt) {

        if (evt.getActionCommand().equals("ok")) {

                cfields = new CheckCalendarFields(mw, form);
                if (!cfields.areFieldsRight()) {
                    cfields.setVisible(true);
                    mw.setEnabled(false);
                }
                else {
                    saveCalendar();
                }

        } else if (evt.getActionCommand().equals("cancel")) {
            mw.show(KEY_CALENDARLIST);
        }
    }

    //-------------------------------------------------------- Protected methods

    /**
     * Sets all the textfields in the DemocCalendar subpanel to an empty string.
     */
    protected void blankFields() {
        form.blankFields();
    }

    //---------------------------------------------------------- Private methods

    /**
     * Saves the new Calendar.
     */
    private void saveCalendar() {
        mw.writeNewCalendar(form.getFields());
        mw.show(KEY_CALENDARLIST);
    }
}
