/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.javagui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import com.funambol.common.pim.contact.Contact;

/**
 * The contact modification panel.
 *
 * @version $Id: ContactModify.java,v 1.7 2007/01/11 19:27:30 nichele Exp $
 */
public class ContactModify 
extends Panel
implements ActionListener, ConfigurationParameters {

    //------------------------------------------------------------- Private data

    //
    // The window containing this panel
    //
    private MainWindow         mw      = null;
    private ContactForm        form    = null;
    private CheckContactFields cfields = null;
    private Language           ln      = new Language();

    //----------------------------------------------------------- Public methods

    /**
     * Creates the panel.
     *
     * @param mw the window containing this panel
     */
    public ContactModify(MainWindow mw) {

        Button butCancel   = null;
        Button butOk       = null;
        Label  title       = null;
        Panel  buttonPanel = null;

        this.mw = mw;

        setLayout(new BorderLayout());
        title = new Label(ln.getString("contact_modify"));

        form = new ContactForm(mw);
        
        butOk = new Button     (ln.getString("ok"));
        butOk.setActionCommand ("ok");
        butOk.addActionListener(this);

        butCancel = new Button     (ln.getString("cancel"));
        butCancel.setActionCommand ("cancel");
        butCancel.addActionListener(this);

        buttonPanel = new Panel();
        buttonPanel.setLayout(new GridLayout(1, 2));
        buttonPanel.add(butOk    );
        buttonPanel.add(butCancel);

        add(title      , BorderLayout.NORTH );
        add(form       , BorderLayout.CENTER);
        add(buttonPanel, BorderLayout.SOUTH );
    }

    /**
     * Invoked when an action occurs (i.e. a button is pressed).
     *
     * @param evt the occurred action
     */
    public void actionPerformed(ActionEvent evt) {       
        if (evt.getActionCommand().equals("ok")) {

            cfields = new CheckContactFields(mw, form);

            if (!cfields.areFieldsRight()) {
                cfields.setVisible(true);
                mw.setEnabled(false);
            } else {
                saveContact();
                form.setInitialTab();
            }

        } else if (evt.getActionCommand().equals("cancel")) {
            mw.show(KEY_CONTACTLIST);
            form.setInitialTab();
        }
    }

    //-------------------------------------------------------- Protected methods

    /**
     * Sets the fields with the values contained in the specified Contact object
     * and the checkbox with the specified type.
     *
     * @param contact the Contact object which is the source of the values
     */
    protected void fillFields(Contact contact) {
        form.setFields(contact);        
    }

    //---------------------------------------------------------- Private methods

    /**
     * Saves the modified contact.
     */
    private void saveContact() {
        mw.writeModContact(form.getFields(),mw.getCurrentIndex().intValue());
        mw.show(KEY_CONTACTLIST);
    }

}
