/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.javagui;

import com.funambol.common.pim.contact.Contact;

/**
 * An object representing a contact.
 * Each contact is identified by its filename and the contact itself in the 
 * form of a Contact object.
 *
 * @see com.funambol.common.pim.contact.Contact
 *
 * @version $Id: DemoContact.java,v 1.5 2007/01/11 19:27:30 nichele Exp $
 */
public class DemoContact {

    //------------------------------------------------------------- Private data
    private Contact contact  = null;
    private String  filename = null;
    private int     type     = 0   ;

    //----------------------------------------------------------- Public methods

    /**
     * Creates a new contact without a specified type
     *
     * @param filename the filename for this contact
     * @param contact the Contact object for this contact
     */
    public DemoContact(String filename, Contact contact) {
        this.filename = filename;
        this.contact = contact;
        this.type = -1;
    }

    /**
     * Creates a new contact
     *
     * @param filename the filename for this contact
     * @param type the type for this contact
     * @param contact the Contact object for this contact
     */
    public DemoContact(String filename, int type, Contact contact) {
        this.filename = filename;
        this.contact = contact;
        this.type = type;
    }

    /**
     * Returns the filename for this contact
     *
     * @return the filename for this contact
     */
    public String getFilename() {
        return filename;
    }

    /**
     * Returns the type for this contact
     *
     * @return the type for this contact
     */
    public int getType() {
        return type;
    }

    /**
     * Returns the Contact object for this contact
     *
     * @return the Contact object for this contact
     */
    public Contact getContact() {
        return contact;
    }

    /**
     * Sets the type of this contact
     *
     * @param type the type to set
     */
    public void setType(int type) {
        this.type = type;
    }

    /**
     * Sets the Contact objecy of this contact
     *
     * @param contact the Contact object to set
     */
    public void setContact(Contact contact) {
        this.contact = contact;
    }

}
