/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.javagui.logging;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import com.funambol.syncclient.common.logging.Handler;

import com.funambol.syncclient.javagui.Language;
import com.funambol.syncclient.javagui.MainWindow;

/**
 * The synchronization panel.
 *
 *
 *
 * @version $Id: PanelHandler.java,v 1.4 2007/01/11 19:27:30 nichele Exp $
 */
public class PanelHandler extends Panel
implements ActionListener, Handler  {

    //---------------------------------------------------------- Constants

    //---------------------------------------------------------- Private data

    //
    // The window containing this panel
    //
    private MainWindow mw      = null           ;

    private TextArea   syncLog = null           ;
    private Button     butOk   = null           ;

    private Language   ln      = new Language() ;

    //---------------------------------------------------------- Public methods


    public void printMessage(String s) {
        log(s);
    }

    /**
     * Creates the panel.
     *
     * @param mw the window containing this panel
     */
    public PanelHandler(MainWindow mw) {

        this.mw = mw;

        setLayout(new BorderLayout());

        syncLog = new TextArea();
        syncLog.setEditable(false);

        butOk = new Button      (ln.getString ("ok") ) ;
        butOk.setActionCommand  ("ok"                ) ;
        butOk.addActionListener (this                ) ;
        butOk.setEnabled        (false               ) ;

        add (syncLog , BorderLayout.CENTER ) ;
        add (butOk   , BorderLayout.SOUTH  ) ;

    }

    /**
     * Invoked when an action occurs (i.e. a button is pressed).
     *
     * @param evt the occurred action
     */
    public void actionPerformed(ActionEvent evt) {
        if (evt.getActionCommand().equals("ok")) {
            syncLog.setText("");
            mw.refresh();
        }
    }

    /**
     * Sets the "Ok" button to the specified value
     *
     * @param value the value to set the button to
     */
    public void setButton(boolean value) {
        butOk.setEnabled(value);
    }

    //---------------------------------------------------------- Protected methods

    /**
     * Shows in the log window the specified message
     *
     * @param msg the message to be shown
     */
    protected void log(String msg) {
        syncLog.append(msg + "\n") ;
    }

    //---------------------------------------------------------- Private methods

}
